#!/bin/bash
ORGWD=`pwd`

#Configurable Parameters
JAR_LOCATION=$ORGWD/outputs/jars #where the JAR files are stored (output from dex2jar)
#The applescript basically opens the "save all" dialog in JD-GUI
#and navigates to the 5th folder you see in the dialog (middle column)
#In my set up this is /opt
SCPT_SAVE_DATA=/opt

#Decompiling
JAVA_TARGET=$ORGWD/outputs/java-output #A folder to store the java code (should not exist)
JDLOCATION=/Applications/JD-GUI.app/Contents/MacOS/jd-gui #JD gui executable
APPLE_SCRIPT=$ORGWD/decompile-Mac-OSX/jd-gui-save-all.scpt #Location of the apple script to do automatic source saving in JD-GUI

########
#Normally you should not need to change anything below here
########

#TODO:
#I know that I could do this script much more elegant if I would use
#a recursive function instead of multiple for loops...

if [ ! -e $JDLOCATION ]
then
    echo "[ERROR] Please install JD-GUI in $JDLOCATION first!"
    echo "[ERROR] This location corresponds to a standard JD-GUI Mac Version installation"
    exit
fi

if [ -e $JAVA_TARGET ]
then
    echo "[ERROR] Please remove $JAVA_TARGET first!"
    exit
else
    mkdir $JAVA_TARGET
fi

function unpackit()
{
    local recursionnr=$1 #e.g. 2
    local foldername=$2
    local folderlocation=$3
    local rstvar=$4
    echo "[INFO] Recursion $recursionnr, doing $folderlocation/$foldername"
    jar -cf $foldername.jar $foldername/
    $JDLOCATION $foldername.jar &
    #The following sleep is necessary, because the apple script will otherwise
    #not realise, that JD-GUI is already running (time depends on CPU scheduler)
    sleep 1.3
    osascript $APPLE_SCRIPT
    local worked="1"
    if [ -e $SCPT_SAVE_DATA/$foldername.src.zip ]
        then
        mkdir $folderlocation/$foldername
        mv $SCPT_SAVE_DATA/$foldername.src.zip $folderlocation/$foldername
        cd $folderlocation/$foldername
        unzip -o -q $foldername.src.zip
        if (( $? ))
            then
            echo "[INFO] Folder $foldername could not be unpacked [iteration $recursionnr]"
            rm -r $folderlocation/$foldername
            worked="0"
        fi
        #rm $foldername.src.zip
        cd -
    else
        echo "[ERROR] The Apple script didn't properly save the zip file, which is even done when JD-GUI crashes (recursion nr $recursionnr)"
    fi
    #rm $foldername.jar
    eval $rstvar="'$worked'"
}


echo "[INFO] Close all JD-GUI windows NOW!"
echo "[INFO] Then remove your hands from the keyboard and mouse and don't touch it anymore"
echo "[INFO] Otherwise this could go pretty wrong"
sleep 5




#Look for the JAR files to decompile/dissassemble
cd $JAR_LOCATION
FILES=`ls *.jar`

for f in $FILES
do
  echo "[INFO] Decompiling $f (to java)..."
  $JDLOCATION $f &
  #The following sleep is necessary, because the apple script will otherwise
  #not realise, that JD-GUI is already running (time depends on CPU scheduler)
  sleep 1.3
  osascript $APPLE_SCRIPT
  filename=${f%.jar}
  echo "FILENAME: $filename"
  if [ -e $SCPT_SAVE_DATA/$filename.src.zip ]
      then
      mkdir $JAVA_TARGET/$filename
      mv $SCPT_SAVE_DATA/$filename.src.zip $JAVA_TARGET/$filename/
      cd $JAVA_TARGET/$filename/
      unzip -o -q $filename.src.zip
      if (( $? ))
          then              
          #If you don't want to handle the "JD-GUI quit unexpectedly" error, the code of this if-clause is unnecessary
          echo "[INFO] The Apple script didn't properly save the zip file"
          echo "[INFO] Therefore unpacking $f and trying every folder on it's own"
          mkdir "extracted-files"
          cd "extracted-files"
          jar xf $JAR_LOCATION/$f
          folders1=`ls`
          for folder1 in $folders1
              do
              result="0"
              unpackit 1 $folder1 $JAVA_TARGET/$filename/ result
              if [[ "0" == "$result" ]]
                  then
                  mkdir $JAVA_TARGET/$filename/$folder1 #We need the parent folder when we are going to move the childs
                  echo "[INFO] Folder $folder1 ist going to be unpacked separately"
                  cd $folder1
                  folders2=`ls`
                  for folder2 in $folders2
                      do
                      unpackit 2 $folder2 $JAVA_TARGET/$filename/$folder1 result
                      if [[ "0" == "$result" ]]
                          then
                          mkdir $JAVA_TARGET/$filename/$folder1/$folder2 #We need the parent folder when we are going to move the childs
                          echo "[INFO] Folder $folder1/$folder2 ist going to be unpacked separately"
                          cd $folder2
                          folders3=`ls`
                          for folder3 in $folders3
                              do
                              unpackit 2 $folder3 $JAVA_TARGET/$filename/$folder1/$folder2 result
                              if [[ "0" == "$result" ]]
                                  then
                                  mkdir $JAVA_TARGET/$filename/$folder1/$folder2/$folder3 #We need the parent folder when we are going to move the childs
                                  echo "[INFO] Folder $folder1/$folder2/$folder3 ist going to be unpacked separately"
                                  cd $folder3
                                  folders4=`ls`
                                  for folder4 in $folders4
                                      do
                                      unpackit 2 $folder4 $JAVA_TARGET/$filename/$folder1/$folder2/$folder3 result
                                      if [[ "0" == "$result" ]]
                                          then
                                          echo "[WARNING] STOPPED! Folder $folder1/$folder2/$folder3/$folder4 ist not going to be unpacked any further"
                                      else
                                          echo "[INFO] $folder1/$folder2/$folder3/$folder4 properly done"
                                          mv $JAVA_TARGET/$filename/extracted-files/$folder1/$folder2/$folder3/$folder4 $JAVA_TARGET/$filename/$folder1/$folder2/$folder3/$folder4
                                        fi
                                  done
                              else
                                  echo "[INFO] $folder1/$folder2/$folder3 properly done"
                                  mv $JAVA_TARGET/$filename/extracted-files/$folder1/$folder2/$folder3 $JAVA_TARGET/$filename/$folder1/$folder2/$folder3
                                fi
                          done
                      else
                          echo "[INFO] $folder1/$folder2 properly done"
                          mv $JAVA_TARGET/$filename/extracted-files/$folder1/$folder2 $JAVA_TARGET/$filename/$folder1/$folder2
                      fi
                  done
              else
                  echo "[INFO] $folder1 properly done"
                  mv $JAVA_TARGET/$filename/extracted-files/$folder1 $JAVA_TARGET/$filename/$folder1 
              fi
              #jar -cf $folder1.jar $folder1/
              #$JDLOCATION $folder1.jar &
              #The following sleep is necessary, because the apple script will otherwise
              #not realise, that JD-GUI is already running (time depends on CPU scheduler)
              #sleep 1
              #osascript $APPLE_SCRIPT
              #if [ -e $SCPT_SAVE_DATA/$folder1.src.zip ]
                  #then
                  #mkdir $JAVA_TARGET/$filename/$folder1
                  #mv $SCPT_SAVE_DATA/$folder1.src.zip $JAVA_TARGET/$filename/$folder1
                  #cd $JAVA_TARGET/$filename/$folder1
                  #unzip -o -q $folder1.src.zip
                  #if (( $? ))
                      #then
                      #echo "[ERROR] Folder $folder1 of $f will be missing, please unpack and decompile separately"
                  #fi
                  #rm $folder1.src.zip
                  #cd -
              #else
            #      echo "[ERROR] The Apple script didn't properly save the zip file, which is even done when JD-GUI crashes (1 recursion)"
              #fi
              #rm $folder1.jar
          done
          #rm -r "$JAVA_TARGET/$filename/extracted-files"
          #rm $JAVA_TARGET/$f/$f.dex2jar.src.zip
          #rm $JAVA_TARGET/$f.dex2jar.jar
      else
          echo "[INFO] $filename properly done"
      fi
  else
      echo "[ERROR] The Apple script didn't properly save the zip file, which is even done when JD-GUI crashes (no recursion yet)"
      echo "[ERROR] Could not find $SCPT_SAVE_DATA/$filename.src.zip"
  fi
  cd $APKLOCATION
done

cd $ORGWD
