#!/bin/bash
ORGWD=`pwd`

#Configurable Parameters
JAR_LOCATION=$ORGWD/outputs/jars #where the JAR files are stored (output from dex2jar)
#The applescript basically opens the "save all" dialog in JD-GUI
#and navigates to the 5th folder you see in the dialog (middle column)
#In my set up this is /opt
SCPT_SAVE_DATA=/opt

#Decompiling
JAVA_TARGET=$ORGWD/outputs/java-output #A folder to store the java code (should not exist)
JDLOCATION=/Applications/JD-GUI.app/Contents/MacOS/jd-gui #JD gui executable
APPLE_SCRIPT=$ORGWD/decompile-Mac-OSX/jd-gui-save-all.scpt #Location of the apple script to do automatic source saving in JD-GUI

########
#Normally you should not need to change anything below here
########

#TODO:
#I know that I could do this script much more elegant if I would use
#a recursive function instead of multiple for loops...

if [ ! -e $JDLOCATION ]
then
    echo "[ERROR] Please install JD-GUI in $JDLOCATION first!"
    echo "[ERROR] This location corresponds to a standard JD-GUI Mac Version installation"
    exit
fi

if [ -e $JAVA_TARGET ]
then
    echo "[ERROR] Please remove $JAVA_TARGET first!"
    exit
else
    mkdir $JAVA_TARGET
fi

function tryToSolveRecursively()
{
    local jartodecompile=$1
    local maxRecursions=$2
    local jarname=${jartodecompile%.jar}
    local result=0
    mkdir $jarname
    cd $jarname
    jar xf $JAR_LOCATION/$jartodecompile
    unpackitrecursive 0 $jarname $JAVA_TARGET $maxRecursions result
    
    if [[ "0" == "$result" ]]
    then
        echo "[INFO] The recursive unpacking did work out, everything alright and done"
    fi
    if [[ "1" == "$result" ]]
    then
        echo "[ERROR] The recursive unpacking did not work out, max recursion depth hit"
    fi
    if [[ "2" == "$result" ]]
    then
        echo "[ERROR] The recursive unpacking did not work out, an unknown error occured"
    fi
    
    cd ..
    rm -r ./extracted-files
}

function unpackitrecursive()
{
    local recursionnr=$1
    local foldername=$2
    local targetlocation=$3
    local maxRecursions=$4
    local rstvar=$5
    local worked="0"
    local folder=""
    
    mkdir $targetlocation/$foldername
    
    if [[ "$recursionnr" == "$maxRecursions" ]] #check if max recursion depth hit
    then
        worked="1"
    else
        folders=`ls -d */`
        for folder in $folders
        do
            $folder=${folder%/}
            echo "[INFO] Recursion $recursionnr, doing $folder in $foldername"
            jar -cf $folder.jar $folder/
            $JDLOCATION $folder.jar &
            #The following sleep is necessary, because the apple script will otherwise
            #not realise, that JD-GUI is already running (time depends on CPU scheduler)
            sleep 1.3
            osascript $APPLE_SCRIPT
            rm ./$folder.jar
            if [ -e $SCPT_SAVE_DATA/$folder.src.zip ]
                then
                mv $SCPT_SAVE_DATA/$folder.src.zip $targetlocation/$foldername/
                cd $targetlocation/$foldername
                unzip -o -q $folder.src.zip
                if (( $? ))
                    then
                    rm $folder.src.zip
                    rm -r ./$folder
                    echo "[INFO] Folder $foldername/$folder could not be unpacked [iteration $recursionnr]"
                    echo "[INFO] Therefore doing a recursive call"
                    recursionnr=$((recursionnr+1))
                    cd -
                    unpackitrecursive $recursionnr $folder $targetlocation/$foldername $maxRecursions worked
                else
                    rm $folder.src.zip
                    echo "[INFO] Successfully unpacked $folder.src.zip"
                    cd -
                    worked="0"
                fi
            else
                echo "[ERROR] The Apple script didn't properly save the $foldername zip file, which is even done when JD-GUI crashes (recursion nr $recursionnr)"
                echo "[ERROR] This is very strange."
                worked="2"
            fi
        done
    fi
    eval $rstvar="'$worked'"
}


echo "[INFO] Close all JD-GUI windows NOW!"
echo "[INFO] Then remove your hands from the keyboard and mouse and don't touch it anymore"
echo "[INFO] Otherwise this could go pretty wrong"
sleep 5

#Look for the JAR files to decompile/dissassemble
cd $JAR_LOCATION
FILES=`ls *.jar`
for f in $FILES
do
  echo "[INFO] Decompiling $f (to java)..."
  #If you don't want to handle the "JD-GUI quit unexpectedly" error, set the maxRecursion to 1
  tryToSolveRecursively $f 5
done

cd $ORGWD
